@@ -143,6 +143,8 @@ static int init(struct ao *ao)
 {
     struct priv *p = ao->priv;
 
+    p->exclusive |= ao->init_flags & AO_INIT_EXCLUSIVE;
+
     if (!af_fmt_is_pcm(ao->format) || p->exclusive) {
         MP_VERBOSE(ao, "redirecting to coreaudio_exclusive\n");
         ao->redirect = "coreaudio_exclusive";
@@ -427,7 +429,9 @@ const struct ao_driver audio_out_coreaudio = {
     .priv_size      = sizeof(struct priv),
     .options = (const struct m_option[]){
         OPT_FLAG("change-physical-format", change_physical_format, 0),
-        OPT_FLAG("exclusive", exclusive, 0),
+        OPT_FLAG("exclusive", exclusive, 0,
+                 .deprecation_message = "use --audio-exclusive"),
         {0}
     },
+    .legacy_prefix = "coreaudio",
 };
