@@ -275,6 +275,8 @@ static int init(struct ao *ao)
     struct wasapi_state *state = ao->priv;
     state->log = ao->log;
 
+    state->opt_exclusive |= ao->init_flags & AO_INIT_EXCLUSIVE;
+
     state->deviceID = wasapi_find_deviceID(ao);
     if (!state->deviceID) {
         uninit(ao);
@@ -495,8 +497,10 @@ const struct ao_driver audio_out_wasapi = {
     .hotplug_uninit = hotplug_uninit,
     .priv_size      = sizeof(wasapi_state),
     .options        = (const struct m_option[]) {
-        OPT_FLAG("exclusive", opt_exclusive, 0),
-        OPT_STRING("device", opt_device, 0),
+        OPT_FLAG("exclusive", opt_exclusive, 0,
+                 .deprecation_message = "use --audio-exclusive"),
+        OPT_STRING("device", opt_device, 0, DEVICE_OPT_DEPRECATION),
         {NULL},
     },
+    .legacy_prefix = "ao-wasapi",
 };
