@@ -21,6 +21,8 @@
 #include <stdbool.h>
 #include <pthread.h>
 
+#include "osdep/atomic.h"
+
 #include "libmpv/client.h"
 
 #include "common/common.h"
@@ -224,6 +226,7 @@ enum playback_status {
 typedef struct MPContext {
     bool initialized;
     bool autodetach;
+    int suspend_count;
     struct mpv_global *global;
     struct MPOpts *opts;
     struct mp_log *log;
@@ -232,6 +235,7 @@ typedef struct MPContext {
     struct mp_client_api *clients;
     struct mp_dispatch_queue *dispatch;
     struct mp_cancel *playback_abort;
+    bool in_dispatch;
 
     struct mp_log *statusline;
     struct osd_state *osd;
@@ -441,6 +445,7 @@ int init_audio_decoder(struct MPContext *mpctx, struct track *track);
 void reinit_audio_chain_src(struct MPContext *mpctx, struct lavfi_pad *src);
 void audio_update_volume(struct MPContext *mpctx);
 void audio_update_balance(struct MPContext *mpctx);
+void reload_audio_output(struct MPContext *mpctx);
 
 // configfiles.c
 void mp_parse_cfgfiles(struct MPContext *mpctx);
@@ -482,7 +487,7 @@ int mp_initialize(struct MPContext *mpctx, char **argv);
 struct MPContext *mp_create(void);
 void mp_destroy(struct MPContext *mpctx);
 void mp_print_version(struct mp_log *log, int always);
-void wakeup_playloop(void *ctx);
+void mp_update_logging(struct MPContext *mpctx, bool preinit);
 
 // misc.c
 double rel_time_to_abs(struct MPContext *mpctx, struct m_rel_time t);
@@ -496,7 +501,6 @@ void error_on_track(struct MPContext *mpctx, struct track *track);
 int stream_dump(struct MPContext *mpctx, const char *source_filename);
 int mpctx_run_reentrant(struct MPContext *mpctx, void (*thread_fn)(void *arg),
                         void *thread_arg);
-struct mpv_global *create_sub_global(struct MPContext *mpctx);
 double get_track_seek_offset(struct MPContext *mpctx, struct track *track);
 
 // osd.c
@@ -510,7 +514,10 @@ void get_current_osd_sym(struct MPContext *mpctx, char *buf, size_t buf_size);
 void set_osd_bar_chapters(struct MPContext *mpctx, int type);
 
 // playloop.c
-void mp_wait_events(struct MPContext *mpctx, double sleeptime);
+void mp_wait_events(struct MPContext *mpctx);
+void mp_set_timeout(struct MPContext *mpctx, double sleeptime);
+void mp_wakeup_core(struct MPContext *mpctx);
+void mp_wakeup_core_cb(void *ctx);
 void mp_process_input(struct MPContext *mpctx);
 double get_relative_time(struct MPContext *mpctx);
 void reset_playback_state(struct MPContext *mpctx);
@@ -536,6 +543,7 @@ void mp_idle(struct MPContext *mpctx);
 void idle_loop(struct MPContext *mpctx);
 int handle_force_window(struct MPContext *mpctx, bool force);
 void seek_to_last_frame(struct MPContext *mpctx);
+void update_screensaver_state(struct MPContext *mpctx);
 
 // scripting.c
 struct mp_scripting {
@@ -543,6 +551,8 @@ struct mp_scripting {
     int (*load)(struct mpv_handle *client, const char *filename);
 };
 void mp_load_scripts(struct MPContext *mpctx);
+void mp_load_builtin_scripts(struct MPContext *mpctx);
+int mp_load_script(struct MPContext *mpctx, const char *fname);
 
 // sub.c
 void reset_subtitle_state(struct MPContext *mpctx);
@@ -569,7 +579,7 @@ void uninit_video_chain(struct MPContext *mpctx);
 double calc_average_frame_duration(struct MPContext *mpctx);
 int init_video_decoder(struct MPContext *mpctx, struct track *track);
 int get_deinterlacing(struct MPContext *mpctx);
-void set_deinterlacing(struct MPContext *mpctx, bool enable);
+void set_deinterlacing(struct MPContext *mpctx, int opt_val);
 
 // Values of MPOpts.softvol
 enum {
