@@ -192,6 +192,7 @@ static void fix_image_params(struct dec_video *d_video,
     struct mp_codec_params *c = d_video->codec;
 
     MP_VERBOSE(d_video, "Decoder format: %s\n", mp_image_params_to_str(params));
+    d_video->dec_format = *params;
 
     // While mp_image_params normally always have to have d_w/d_h set, the
     // decoder signals unknown bitstream aspect ratio with both set to 0.
@@ -236,6 +237,13 @@ static void fix_image_params(struct dec_video *d_video,
     if (p.p_w <= 0 || p.p_h <= 0)
         p.p_w = p.p_h = 1;
 
+    if (opts->video_rotate < 0) {
+        p.rotate = 0;
+    } else {
+        p.rotate = (p.rotate + opts->video_rotate) % 360;
+    }
+    p.stereo_out = opts->video_stereo_mode;
+
     // Detect colorspace from resolution.
     mp_image_params_guess_csp(&p);
 
@@ -341,11 +349,16 @@ static struct mp_image *decode_packet(struct dec_video *d_video,
     return mpi;
 }
 
-void video_reset_aspect(struct dec_video *d_video)
+void video_reset_params(struct dec_video *d_video)
 {
     d_video->last_format = (struct mp_image_params){0};
 }
 
+void video_get_dec_params(struct dec_video *d_video, struct mp_image_params *p)
+{
+    *p = d_video->dec_format;
+}
+
 void video_set_framedrop(struct dec_video *d_video, bool enabled)
 {
     d_video->framedrop_enabled = enabled;
@@ -364,16 +377,16 @@ void video_work(struct dec_video *d_video)
 
     if (d_video->header->attached_picture) {
         if (d_video->current_state == DATA_AGAIN && !d_video->cover_art_mpi) {
-            d_video->cover_art_mpi =
-                decode_packet(d_video, d_video->header->attached_picture, 0);
+            struct demux_packet *packet =
+                demux_copy_packet(d_video->header->attached_picture);
+            d_video->cover_art_mpi = decode_packet(d_video, packet, 0);
             // Might need flush.
             if (!d_video->cover_art_mpi)
                 d_video->cover_art_mpi = decode_packet(d_video, NULL, 0);
-            d_video->current_state = DATA_OK;
+            talloc_free(packet);
         }
-        if (d_video->current_state == DATA_OK)
+        if (d_video->current_state != DATA_EOF)
             d_video->current_mpi = mp_image_new_ref(d_video->cover_art_mpi);
-        // (DATA_OK is returned the first time, when current_mpi is sill set)
         d_video->current_state = DATA_EOF;
         return;
     }
