@@ -30,7 +30,7 @@
 #import "video/out/cocoa/mpvadapter.h"
 
 #include "osdep/threads.h"
-#include "osdep/atomics.h"
+#include "osdep/atomic.h"
 #include "osdep/macosx_compat.h"
 #include "osdep/macosx_events_objc.h"
 
@@ -90,8 +90,6 @@ struct vo_cocoa_state {
     uint32_t old_dwidth;
     uint32_t old_dheight;
 
-    NSData *icc_wnd_profile;
-    NSData *icc_fs_profile;
     id   fs_icc_changed_ns_observer;
 
     pthread_mutex_t lock;
@@ -421,7 +419,6 @@ static void vo_cocoa_update_screen_info(struct vo *vo, struct mp_rect *out_rc)
         return;
 
     vo_cocoa_update_screens_pointers(vo);
-    vo_cocoa_update_screen_fps(vo);
 
     if (out_rc) {
         NSRect r = [s->current_screen frame];
@@ -452,7 +449,6 @@ static int vo_cocoa_ontop(struct vo *vo)
         return VO_NOTIMPL;
 
     struct mp_vo_opts *opts = vo->opts;
-    opts->ontop = !opts->ontop;
     vo_set_level(vo, opts->ontop);
     return VO_TRUE;
 }
@@ -462,10 +458,10 @@ static MpvVideoWindow *create_window(NSRect rect, NSScreen *s, bool border,
 {
     int window_mask = 0;
     if (border) {
-        window_mask = NSTitledWindowMask|NSClosableWindowMask|
-                      NSMiniaturizableWindowMask|NSResizableWindowMask;
+        window_mask = NSWindowStyleMaskTitled|NSWindowStyleMaskClosable|
+                      NSWindowStyleMaskMiniaturizable|NSWindowStyleMaskResizable;
     } else {
-        window_mask = NSBorderlessWindowMask|NSResizableWindowMask;
+        window_mask = NSWindowStyleMaskBorderless|NSWindowStyleMaskResizable;
     }
 
     MpvVideoWindow *w =
@@ -584,6 +580,7 @@ static void cocoa_screen_reconfiguration_observer(
         struct vo *vo = ctx;
         MP_WARN(vo, "detected display mode change, updating screen info\n");
         vo_cocoa_update_screen_info(vo, NULL);
+        vo_cocoa_update_screen_fps(vo);
     }
 }
 
@@ -614,6 +611,7 @@ int vo_cocoa_config_window(struct vo *vo)
     run_on_main_thread(vo, ^{
         struct mp_rect screenrc;
         vo_cocoa_update_screen_info(vo, &screenrc);
+        vo_cocoa_update_screen_fps(vo);
 
         struct vo_win_geometry geo;
         vo_calc_window_geometry(vo, &screenrc, &geo);
@@ -720,8 +718,6 @@ void vo_cocoa_swap_buffers(struct vo *vo)
     if (skip)
         return;
 
-    CGLFlushDrawable(s->cgl_ctx);
-
     pthread_mutex_lock(&s->lock);
     s->frame_w = vo->dwidth;
     s->frame_h = vo->dheight;
@@ -794,7 +790,6 @@ static int vo_cocoa_control_on_main_thread(struct vo *vo, int request, void *arg
 
     switch (request) {
     case VOCTRL_FULLSCREEN:
-        opts->fullscreen = !opts->fullscreen;
         return vo_cocoa_fullscreen(vo);
     case VOCTRL_ONTOP:
         return vo_cocoa_ontop(vo);
@@ -953,6 +948,7 @@ int vo_cocoa_control(struct vo *vo, int *events, int request, void *arg)
 - (void)windowDidChangeScreen:(NSNotification *)notification
 {
     vo_cocoa_update_screen_info(self.vout, NULL);
+    vo_cocoa_update_screen_fps(self.vout);
 }
 
 - (void)didChangeWindowedScreenProfile:(NSScreen *)screen
