@@ -16,6 +16,8 @@ struct gl_hwdec {
     void *priv;
     // For working around the vdpau vs. vaapi mess.
     bool probing;
+    // Used in overlay mode only.
+    float overlay_colorkey[4];
 };
 
 struct gl_hwdec_plane {
@@ -36,6 +38,7 @@ struct gl_hwdec_driver {
     // Used to explicitly request a specific API.
     enum hwdec_type api;
     // The hardware surface IMGFMT_ that must be passed to map_image later.
+    // If the test_format callback is set, this field is ignored!
     int imgfmt;
     // Create the hwdec device. It must add it to hw->devs, if applicable.
     int (*create)(struct gl_hwdec *hw);
@@ -53,6 +56,24 @@ struct gl_hwdec_driver {
     void (*unmap)(struct gl_hwdec *hw);
 
     void (*destroy)(struct gl_hwdec *hw);
+
+    // Optional callback for checking input format support.
+    bool (*test_format)(struct gl_hwdec *hw, int imgfmt);
+
+    // The following functions provide an alternative API. Each gl_hwdec_driver
+    // must have either map_frame or overlay_frame set (not both or none), and
+    // if overlay_frame is set, it operates in overlay mode. In this mode,
+    // OSD etc. is rendered via OpenGL, but the video is rendered as a separate
+    // layer below it.
+    // Non-overlay mode is strictly preferred, so try not to use overlay mode.
+
+    // Set the given frame as overlay, replacing the previous one.
+    // hw_image==NULL is passed to clear the overlay.
+    int (*overlay_frame)(struct gl_hwdec *hw, struct mp_image *hw_image);
+
+    // Move overlay position within the "window".
+    void (*overlay_adjust)(struct gl_hwdec *hw, int w, int h,
+                           struct mp_rect *src, struct mp_rect *dst);
 };
 
 struct gl_hwdec *gl_hwdec_load_api(struct mp_log *log, GL *gl,
@@ -62,4 +83,6 @@ struct gl_hwdec *gl_hwdec_load_api(struct mp_log *log, GL *gl,
 
 void gl_hwdec_uninit(struct gl_hwdec *hwdec);
 
+bool gl_hwdec_test_format(struct gl_hwdec *hwdec, int imgfmt);
+
 #endif
