@@ -1087,8 +1087,6 @@ static int control(struct vo *vo, uint32_t request, void *data)
     check_preemption(vo);
 
     switch (request) {
-    case VOCTRL_GET_PANSCAN:
-        return VO_TRUE;
     case VOCTRL_SET_PANSCAN:
         checked_resize(vo);
         return VO_TRUE;
@@ -1139,6 +1137,8 @@ const struct vo_driver video_out_vdpau = {
     .control = control,
     .draw_frame = draw_frame,
     .flip_page = flip_page,
+    .wakeup = vo_x11_wakeup,
+    .wait_events = vo_x11_wait_events,
     .uninit = uninit,
     .priv_size = sizeof(struct vdpctx),
     .options = (const struct m_option []){
@@ -1150,15 +1150,19 @@ const struct vo_driver video_out_vdpau = {
         OPT_INTRANGE("hqscaling", hqscaling, 0, 0, 9),
         OPT_FLOAT("fps", user_fps, 0),
         OPT_FLAG("composite-detect", composite_detect, 0, OPTDEF_INT(1)),
-        OPT_INT("queuetime_windowed", flip_offset_window, 0, OPTDEF_INT(50)),
-        OPT_INT("queuetime_fs", flip_offset_fs, 0, OPTDEF_INT(50)),
-        OPT_INTRANGE("output_surfaces", num_output_surfaces, 0,
+        OPT_INT("queuetime-windowed", flip_offset_window, 0, OPTDEF_INT(50)),
+        OPT_INT("queuetime-fs", flip_offset_fs, 0, OPTDEF_INT(50)),
+        OPT_INTRANGE("output-surfaces", num_output_surfaces, 0,
                      2, MAX_OUTPUT_SURFACES, OPTDEF_INT(3)),
         OPT_COLOR("colorkey", colorkey, 0,
                   .defval = &(const struct m_color) {
                       .r = 2, .g = 5, .b = 7, .a = 255,
                   }),
         OPT_FLAG("force-yuv", force_yuv, 0),
+        OPT_REPLACED("queuetime_windowed", "queuetime-windowed"),
+        OPT_REPLACED("queuetime_fs", "queuetime-fs"),
+        OPT_REPLACED("output_surfaces", "output-surfaces"),
         {NULL},
-    }
+    },
+    .legacy_prefix = "vo-vdpau",
 };
